## Copyright 2007 Tom Brown

## This program is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 3 of the
## License, or (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

## See http://www.nworbmot.org/code/mult.html for more usage
## instructions




def sizePart(partition):
    size = 0
    for i in partition:
        size = size + i
    return size  


# this assumes that permutation is in the Partition class
def character(partition,permutation):
    if partition == "nat":
        return permutation.to_list().count(1)
    else:
        size = sizePart(partition)
        all_partitions = [p for p in Partitions(size)]
        return symmetrica.chartafel(size)[all_partitions.index(partition),all_partitions.index(permutation)]




#Counts the multiplicity of V_{\L_1}^{S_n} \otimes V_{\L_2}^{S_k} in
#V_{[n-1,1]}^{\otimes k}

def multiplicity(Lambda_1,Lambda_2):
    n = sizePart(Lambda_1)
    k = sizePart(Lambda_2)

    Hook = [n-1,1]

    result = 0

    for p_n in Partitions(n):
            for p_k in Partitions(k):
                productOfTraces = 1
                for i in range(0,len(p_k)):
                        productOfTraces = productOfTraces*character(Hook,partition_power(p_n,p_k[i]))
                result=result +productOfTraces*character(Lambda_1,p_n)*character(Lambda_2,p_k)*p_n.conjugacy_class_size()*p_k.conjugacy_class_size()

    result = result/(factorial(n)*factorial(k))

    return result

# see http://www.sagemath.org/doc/html/ref/module-sage.combinat.partition.html
def hook_product(Lambda):
    result = 1
    for index,item in enumerate(Partition(Lambda).hooks()):
    	result = result*item
        
    return result





#Counts the multiplicity of V_{\L_1}^{S_n} \otimes V_{\L_2}^{S_k} in
#V_{Hook}^{\otimes k} where Hook can be anything

def super_multiplicity(Hook,Lambda_1,Lambda_2):
    n = sizePart(Lambda_1)
    k = sizePart(Lambda_2)

    result = 0

    for p_n in Partitions(n):
            for p_k in Partitions(k):
                productOfTraces = 1
                for i in range(0,len(p_k)):
                        productOfTraces = productOfTraces*character(Hook,p_n.power(p_k[i]))
#                        productOfTraces = productOfTraces*character(Hook,partition_power(p_n,p_k[i]))
                result=result +productOfTraces*character(Lambda_1,p_n)*character(Lambda_2,p_k)*p_n.conjugacy_class_size()*p_k.conjugacy_class_size()

    result = result/(factorial(n)*factorial(k))

    return result



#Counts the multiplicity of V_{\L_1}^{S_n} in
#V_{Hook}^{\otimes k} where Hook can be anything

def partition_multiplicity(Hook,Lambda_1,k):
    n = sizePart(Lambda_1)

    result = 0

    for p_n in Partitions(n):
        result = result + character(Lambda_1,p_n)*p_n.conjugacy_class_size()*(character(Hook,p_n))^k

    result = result/(factorial(n))

    return result




# see http://www.sagemath.org/doc/html/ref/module-sage.combinat.partition.html
def hook_product(Lambda):
    result = 1
    for index,item in enumerate(Partition(Lambda).hooks()):
    	result = result*item
        
    return result





def sym_dim(Lambda):
    n = sizePart(Lambda)
    return factorial(n)/hook_product(Lambda)

def gl_dim(Lambda, N):
    result = 1
    for (i,j) in Partition(Lambda).boxes():
        result = result*(N +j-i)
    return factor(result/hook_product(Lambda))

def gl_dim_yak(Lambda, N):
    result = 1
    for (i,j) in Partition(Lambda).boxes():
        result = result*(N +j-i)
    return result/hook_product(Lambda)




# computes the symmetric group tensor product C(R,S,T)
def inner(R,S,T):

    n = sizePart(R)
    
    result = 0

    for perm in Partitions(n):
        result = result + perm.conjugacy_class_size()*character(R,perm)*character(S,perm)*character(T,perm)

    result = result/factorial(n)

    return result



# This function takes the symmetric group reps R and S and returns an
# array with the coefficients of their inner product

def inner_ret_array(R,S):
    
    n = sizePart(R)

    result = []

    for p in Partitions(n):
        dummy = inner(R,S,p)
        if dummy > 0:
            result.append([dummy,p])

    return result

##adds array 1 to array2
def add_arrays(array1,array2):

    for i in range(0,len(array1)):
        dummy = 0
        for j in range(0,len(array2)):
            if array1[i][1]==array2[j][1]:
                array2[j][0] = array2[j][0]+array1[i][0]
                dummy =1
                break
        if dummy == 0:
            array2.append(array1[i])

    return array2


def mult_int(integer,array):
    for i in range(0,len(array)):
        array[i][0] = integer*array[i][0]

    return array


                              
def inner_arrays(array1,array2):

    result=[]

    for i in range(0,len(array1)):
        for j in range(0,len(array2)):   
            add_arrays(mult_int(array1[i][0]*array2[j][0],inner_ret_array(array1[i][1],array2[j][1])),result)


    return result


# this function is no faster than the above, but it does put the partitions in order

def inner_arrays2(array1,array2):

    result=[]

    n = sizePart(array1[0][1])

    for p in Partitions(n):
        sum = 0
        for i in range(0,len(array1)):
            for j in range(0,len(array2)):
                sum = sum + array1[i][0]*array2[j][0]*inner(array1[i][1],array2[j][1],p)
        if sum > 0:
            print sum,p
            result.append([sum,p])


    return result

    
                       

#returns an array of the Lambda_1 reps of S_n in Lambda_2(Hook^k)

def super_multiplicity_array(Hook,Lambda_2,n):

    result = []
    for Lambda_1 in Partitions(n):
        dummy = super_multiplicity(Hook,Lambda_1,Lambda_2)
        if dummy > 0:
            result.append([dummy,Lambda_1])


    return result

#returns an array of the Lambda_1 reps of S_n in [array](Hook^k)

def super_multiplicity_arrays(Hook,array,n):

    overall_result = []

    for i in range(0,len(array)):
        result=[]
        for Lambda_1 in Partitions(n):
            dummy = super_multiplicity(Hook,Lambda_1,array[i][1])
            if dummy > 0:
                result.append([dummy*array[i][0],Lambda_1])
        add_arrays(result,overall_result)

    return overall_result




# takes two partitions and then the maximum size of the two partitions
def add_perms(perm_1,perm_2,max):

    result = []
    for i in range(max,0,-1):
        number = perm_1.to_list().count(i) + perm_2.to_list().count(i)
        if number >0:
            result = result +[i]*number

    return Partition(result)

# computes the Littlewood-Richardson coefficient g(R,S;T), 
# takes three lists or partitions
def lr(R,S,T):
    
    r = sizePart(R)
    s = sizePart(S)
    t = sizePart(T)

    max = r

    if s > max:
        max = s

    if r+s!=t:
        print "error - t is not r+s"
        return 0


    result = 0
    for perm_r in Partitions(r):
        for perm_s in Partitions(s):
            result = result + perm_r.conjugacy_class_size()*perm_s.conjugacy_class_size()*character(R,perm_r)*character(S,perm_s)*character(T,add_perms(perm_r,perm_s,max))

    return result/(factorial(r)*factorial(s))




# computes the Littlewood-Richardson coefficient g(R,S), 
# takes two lists or partitions and returns an array
def lr_ret_array(R,S):
    
    r = sizePart(R)
    s = sizePart(S)

    array = []

    max = r

    if s > max:
        max = s


    for T in Partitions(r+s):

        result = 0
        for perm_r in Partitions(r):
            for perm_s in Partitions(s):
                result = result + perm_r.conjugacy_class_size()*perm_s.conjugacy_class_size()*character(R,perm_r)*character(S,perm_s)*character(T,add_perms(perm_r,perm_s,max))


        if result > 0:
            array.append([result/(factorial(r)*factorial(s)),T])

    return array
